@@ -7,6 +7,17 @@ SSH agent-based authentication for github2gerrit.
 This module provides functionality to use SSH agent for authentication
 instead of writing private keys to disk, which is more secure and
 avoids file permission issues in CI environments.
+
+SSH Agent Validation Strategy:
+- Early CLI validation defers SSH agent checks to avoid duplicate validation
+- Runtime validation in setup_ssh_agent_auth() performs comprehensive checks:
+  1. Checks for existing SSH agent (SSH_AUTH_SOCK environment variable)
+  2. Validates agent has keys loaded using "ssh-add -l" command
+  3. Gracefully falls back to file-based SSH if agent validation fails
+  4. Supports starting new agents when private keys are provided
+
+This approach ensures robust SSH authentication while maintaining clean
+separation between configuration validation and runtime validation.
 """
 
 from __future__ import annotations
@@ -44,15 +55,9 @@ _MSG_NO_KEYS_LOADED = "No keys were loaded into SSH agent"
 _MSG_SSH_AGENT_NOT_FOUND = "ssh-agent not found in PATH"
 _MSG_SSH_ADD_NOT_FOUND = "ssh-add not found in PATH"
 _MSG_TOOL_NOT_FOUND = "Required tool '{tool_name}' not found in PATH"
-_MSG_WORKSPACE_NOT_SET = "Workspace not set"
 _MSG_NO_AGENT_AND_KEY = "No SSH agent and no key provided"
 
 
-def _raise_workspace_error() -> None:
-    """Raise error when workspace is not set."""
-    raise SSHAgentError(_MSG_WORKSPACE_NOT_SET)
-
-
 def _raise_no_keys_error() -> None:
     """Raise error when no keys are loaded."""
     raise SSHAgentError(_MSG_NO_KEYS_LOADED)
@@ -66,18 +71,23 @@ def _raise_no_agent_error() -> None:
 class SSHAgentManager:
     """Manages SSH agent lifecycle and key loading for secure authentication."""
 
-    def __init__(self, workspace: Path | None = None):
+    def __init__(self, workspace: Path):
         """Initialize SSH agent manager.
 
+        This class manages both SSH agent lifecycle and secure file storage.
+        All instances require a workspace for consistent behavior.
+
         Args:
             workspace: Secure temporary directory for storing SSH files (outside
-                git workspace). Optional when using existing agent.
+                git workspace). Required for all operations including
+                known_hosts management and secure cleanup.
         """
         self.workspace = workspace
         self.agent_pid: int | None = None
         self.auth_sock: str | None = None
         self.known_hosts_path: Path | None = None
         self._original_env: dict[str, str] = {}
+        self._agent_owned_by_us: bool = False
 
     def start_agent(self) -> None:
         """Start a new SSH agent process."""
@@ -99,6 +109,7 @@ class SSHAgentManager:
                     # Format: SSH_AGENT_PID=12345; export SSH_AGENT_PID;
                     value = line.split("=", 1)[1].split(";")[0].strip()
                     self.agent_pid = int(value)
+                    self._agent_owned_by_us = True  # We started this agent
 
             if not self.auth_sock or not self.agent_pid:
                 _raise_parse_error()
@@ -164,7 +175,13 @@ class SSHAgentManager:
 
             # Store the existing agent info
             self.auth_sock = auth_sock
-            self.agent_pid = None  # We don't own this agent
+            # Try to get PID from environment for informational purposes only
+            agent_pid_str = os.environ.get("SSH_AGENT_PID")
+            if agent_pid_str and agent_pid_str.isdigit():
+                self.agent_pid = int(agent_pid_str)
+            else:
+                self.agent_pid = None
+            self._agent_owned_by_us = False  # We're borrowing this agent
 
             log.debug(
                 "Successfully connected to existing SSH agent: %s", auth_sock
@@ -201,11 +218,7 @@ class SSHAgentManager:
                 stderr=subprocess.PIPE,
                 text=True,
                 shell=False,  # Explicitly disable shell for security
-                env={
-                    **os.environ,
-                    "SSH_AUTH_SOCK": self.auth_sock,
-                    "SSH_AGENT_PID": str(self.agent_pid),
-                },
+                env=self._get_ssh_env(),
             )
 
             _stdout, stderr = process.communicate(
@@ -234,9 +247,7 @@ class SSHAgentManager:
             # Create tool-specific SSH directory in secure temp location
             # Note: workspace is now a separate secure temp directory outside
             # git workspace
-            if self.workspace is None:
-                _raise_workspace_error()
-            workspace = cast(Path, self.workspace)  # Type narrowing for mypy
+            workspace = self.workspace
             tool_ssh_dir = workspace / ".ssh-g2g"
             tool_ssh_dir.mkdir(mode=0o700, exist_ok=True)
 
@@ -275,8 +286,6 @@ class SSHAgentManager:
         if not self.known_hosts_path:
             raise SSHAgentError(_MSG_HOSTS_NOT_CONFIGURED)
 
-        import os
-
         # Check if we should respect user SSH config
         respect_user_ssh = os.getenv(
             "G2G_RESPECT_USER_SSH", "false"
@@ -302,8 +311,8 @@ class SSHAgentManager:
 
         return f"ssh {' '.join(ssh_options)}"
 
-    def get_ssh_env(self) -> dict[str, str]:
-        """Get environment variables for SSH operations.
+    def _get_ssh_env(self) -> dict[str, str]:
+        """Get SSH environment variables for internal subprocess calls.
 
         Returns:
             Dictionary of environment variables
@@ -311,16 +320,47 @@ class SSHAgentManager:
         if not self.auth_sock:
             raise SSHAgentError(_MSG_NOT_STARTED)
 
-        return {
+        env = {
+            **os.environ,
             "SSH_AUTH_SOCK": self.auth_sock,
-            "SSH_AGENT_PID": str(self.agent_pid),
         }
 
-    def list_keys(self) -> str:
-        """List keys currently loaded in the agent.
+        # Only set SSH_AGENT_PID if we have one (borrowed agents might not)
+        if self.agent_pid is not None:
+            env["SSH_AGENT_PID"] = str(self.agent_pid)
+
+        return env
+
+    def get_ssh_env(self) -> dict[str, str]:
+        """Get SSH environment variables for subprocess calls.
 
         Returns:
-            Output from ssh-add -l
+            Dictionary of environment variables
+        """
+        if not self.auth_sock:
+            raise SSHAgentError(_MSG_NOT_STARTED)
+
+        env = {"SSH_AUTH_SOCK": self.auth_sock}
+
+        # Only set SSH_AGENT_PID if we have one (borrowed agents might not)
+        if self.agent_pid is not None:
+            env["SSH_AGENT_PID"] = str(self.agent_pid)
+
+        return env
+
+    def list_keys(self) -> str:
+        """List keys currently loaded in the agent using 'ssh-add -l'.
+
+        This method performs the actual SSH agent validation by executing
+        'ssh-add -l' to check if any SSH keys are loaded in the agent.
+        It's used throughout the SSH setup process to validate agent state
+        and ensure SSH operations will succeed.
+
+        Returns:
+            Output from ssh-add -l, or "No keys loaded" if agent has no keys
+
+        Raises:
+            SSHAgentError: If SSH agent is not started or ssh-add fails
         """
         if not self.auth_sock:
             raise SSHAgentError(_MSG_NOT_STARTED)
@@ -331,11 +371,7 @@ class SSHAgentManager:
 
             result = run_cmd(
                 [ssh_add_path, "-l"],
-                env={
-                    **os.environ,
-                    "SSH_AUTH_SOCK": self.auth_sock,
-                    "SSH_AGENT_PID": str(self.agent_pid),
-                },
+                env=self._get_ssh_env(),
                 timeout=5,
             )
         except CommandError as exc:
@@ -350,16 +386,23 @@ class SSHAgentManager:
     def cleanup(self) -> None:
         """Securely clean up SSH agent and temporary files."""
         try:
-            # Only kill SSH agent if we started it (agent_pid is set)
+            # Only kill SSH agent if we started it ourselves
             # Never kill an existing SSH agent that we're borrowing
-            if self.agent_pid:
+            if self.agent_pid and self._agent_owned_by_us:
                 try:
                     run_cmd(["/bin/kill", str(self.agent_pid)], timeout=5)
                     log.debug("SSH agent (PID %d) terminated", self.agent_pid)
                 except Exception as exc:
                     log.warning("Failed to kill SSH agent: %s", exc)
             else:
-                log.debug("Not terminating SSH agent (using existing agent)")
+                if self.agent_pid and not self._agent_owned_by_us:
+                    log.debug(
+                        "Not terminating SSH agent (borrowed existing agent "
+                        "with PID %d)",
+                        self.agent_pid,
+                    )
+                else:
+                    log.debug("Not terminating SSH agent (no agent running)")
 
             # Restore original environment
             for key, value in self._original_env.items():
@@ -369,8 +412,6 @@ class SSHAgentManager:
                     del os.environ[key]
 
             # Securely clean up temporary files
-            if self.workspace is None:
-                return
             tool_ssh_dir = self.workspace / ".ssh-g2g"
             if tool_ssh_dir.exists():
                 import shutil
@@ -416,25 +457,40 @@ class SSHAgentManager:
             self.agent_pid = None
             self.auth_sock = None
             self.known_hosts_path = None
+            self._agent_owned_by_us = False
 
 
 def setup_ssh_agent_auth(
     workspace: Path, private_key_content: str, known_hosts_content: str
 ) -> SSHAgentManager:
-    """Setup SSH agent-based authentication.
+    """Setup SSH agent-based authentication with comprehensive validation.
+
+    This function performs the runtime SSH agent validation that was deferred
+    from early CLI validation to avoid duplicate checks. It implements a
+    robust validation and fallback strategy:
+
+    1. Check for existing SSH agent (SSH_AUTH_SOCK environment variable)
+    2. Validate existing agent has keys loaded using "ssh-add -l"
+    3. If existing agent has no keys but private key provided, start new agent
+    4. If no existing agent, start new agent with provided private key
+    5. Fail gracefully if no agent available and no private key provided
+
+    The validation using "ssh-add -l" ensures that SSH operations will succeed
+    and helps prevent authentication failures during Git operations.
 
     Args:
         workspace: Secure temporary directory for SSH files (outside git
             workspace)
         private_key_content: SSH private key content (empty string to use
-            existing agent)
-        known_hosts_content: Known hosts content
+            existing agent only)
+        known_hosts_content: Known hosts content for SSH host verification
 
     Returns:
-        Configured SSH agent manager
+        SSHAgentManager instance for the configured agent
 
     Raises:
-        SSHAgentError: If setup fails
+        SSHAgentError: If SSH agent setup fails (no agent + no key, or
+                      validation failures)
     """
     manager = SSHAgentManager(workspace)
 
@@ -450,7 +506,8 @@ def setup_ssh_agent_auth(
             # Setup known hosts for existing agent
             manager.setup_known_hosts(known_hosts_content)
 
-            # Verify existing agent has keys
+            # Verify existing agent has keys using "ssh-add -l"
+            # This validation ensures SSH operations will succeed
             keys_list = manager.list_keys()
             if "No keys loaded" in keys_list:
                 log.debug("Existing SSH agent has no keys loaded")
@@ -489,7 +546,8 @@ def setup_ssh_agent_auth(
         # Setup known hosts
         manager.setup_known_hosts(known_hosts_content)
 
-        # Verify key was added
+        # Verify key was added successfully using "ssh-add -l"
+        # This final validation ensures the agent is ready for SSH operations
         keys_list = manager.list_keys()
         if "No keys loaded" in keys_list:
             _raise_no_keys_error()
